<?php

namespace Drupal\imago\Plugin\views\query;

use Drupal\Core\Session\AccountProxyInterface;
use Drupal\views\Plugin\views\query\QueryPluginBase;
use Drupal\views\ResultRow;
use Drupal\views\ViewExecutable;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Imago views query plugin.
 *
 * @ViewsQuery(
 *   id = "imago",
 *   title = @Translation("Imago"),
 *   help = @Translation("Query against the Imago API.")
 * )
 */
class ImagoQueryPluginBase extends QueryPluginBase {

  /**
   * Current user ID.
   *
   * @var int
   */
  protected $currentUser;

  /**
   * Constructs a PluginBase object.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Session\AccountProxyInterface $current_user
   *   Current user account.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, AccountProxyInterface $current_user) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->currentUser = $current_user;
    $this->definition = $plugin_definition + $configuration;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('current_user')
    );
  }

  /**
   * Ensure a table exists in the queue.
   *
   * @param string|null $table
   *   The unaliased name of the table to ensure.
   * @param string|null $relationship
   *   The relationship to ensure the table links to.
   *
   * @return string|null
   *   The alias used to refer to this specific table, or NULL if the table
   *   cannot be ensured.
   */
  public function ensureTable($table, $relationship = NULL) {
    return '';
  }

  /**
   * Add a field to the query table.
   *
   * This will automatically call ensureTable to make sure the
   * required table exists, *unless* $table is unset.
   *
   * @param string|null $table
   *   The table this field is attached to. If NULL, it is assumed this will
   *   be a formula; otherwise, ensureTable is used to make sure the
   *   table exists.
   * @param string $field
   *   The name of the field to add. This may be a real field or a formula.
   * @param string|null $alias
   *   The alias to create. If not specified, the alias will be $table_$field
   *   unless $table is NULL. When adding formulae, it is recommended that an
   *   alias be used.
   * @param array $params
   *   An array of parameters additional to the field that will control items
   *   such as aggregation functions and DISTINCT. Some values that are
   *   recognized:
   *   - function: An aggregation function to apply, such as SUM.
   *   - aggregate: Set to TRUE to indicate that this value should be
   *     aggregated in a GROUP BY.
   *
   * @return string
   *   The name that this field can be referred to as.
   */
  public function addField($table, $field, $alias = '', array $params = []) {
    return $field;
  }

  /**
   * {@inheritdoc}
   */
  public function execute(ViewExecutable $view) {
    $query_config = \Drupal::config('imago.settings');
    $config = $query_config->get('api');
    $imago_manager = \Drupal::service('imago.manager');
    $exposed_data = $view->exposed_data;
    $filters = [];

    if (!empty($exposed_data['caption'])) {
      $filters['querystring'] = $exposed_data['caption'];
    }
    if (!empty($exposed_data['limit']) && is_numeric($exposed_data['limit'])) {
      $filters['size'] = $exposed_data['limit'];
      $total_rows = \Drupal::state()->get('imago.page.' . $this->currentUser->id());
      if (!empty($total_rows) && is_integer($total_rows['count']) && $exposed_data['limit'] !== $total_rows['count']) {
        $filters['from'] = $total_rows['count'];
      }
    }
    if (!empty($exposed_data['database'])) {
      $filters['db'] = $exposed_data['database'];
    }
    if (!empty($filters)) {
      $filters['sortby'] = 'relevanz';
      $filters['sort'] =  'desc';
    }

    $images = $imago_manager->connectSearch($filters);
    $index = 0;
    $results = [];
    foreach ($images as $image) {
      $database = $this->getDatabaseId($image['db']);
      $row['caption'] = $image['caption'];
      $row['thumbnail'] = $config['api_url']['base_url'] . '/' . $database . '/' . $image['pictureid'] . '/thumbs';
      $row['picture_id'] = $image['pictureid'];
      $row['source'] = $image['source'];
      $row['created'] = $image['datecreated'];
      $row['archival_date'] = $image['archivaldate'];
      $row['database'] = $image['db'];
      $row['height'] = $image['height'];
      $row['width'] = $image['width'];
      $row['index'] = $index++;
      $view->result[] = new ResultRow($row);
      $results[$image['pictureid']] = $row;
    }
    \Drupal::state()->set('imago.api_query.' . $this->currentUser->id(), $results);
  }

  /**
   * Add a simple WHERE clause to the query.
   *
   * The caller is responsible for ensuring that all fields are fully
   * qualified (TABLE.FIELD)    * and that the table already exists in
   * the query.
   *
   * The $field, $value and $operator arguments can also be passed in with a
   * single DatabaseCondition object, like this:
   * @code
   * $this->query->addWhere(
   *   $this->options['group'],
   *   (new Condition('OR'))
   *     ->condition($field, $value, 'NOT IN')
   *     ->condition($field, $value, 'IS NULL')
   * );
   * @endcode
   *
   * @param int $group
   *   The WHERE group to add these to; groups are used to create AND/OR
   *   sections. Groups cannot be nested. Use 0 as the default group.
   *   If the group does not yet exist it will be created as an AND group.
   * @param string $field
   *   The name of the field to check.
   * @param null|string $value
   *   The value to test the field against. In most cases, this is a scalar.
   *   For more complex options, it is an array. The meaning of each element
   *   in the array is dependent on the $operator.
   * @param null|string $operator
   *   The comparison operator, such as =, <, or >=. It also accepts more
   *   complex options such as IN, LIKE, LIKE BINARY, or BETWEEN.
   *   Defaults to =. If $field is a string you have to use 'formula' here.
   *
   * @see \Drupal\Core\Database\Query\ConditionInterface::condition()
   * @see \Drupal\Core\Database\Query\Condition
   */
  public function addWhere($group, $field, $value = NULL, $operator = NULL) {
    // Ensure all variants of 0 are actually 0. Thus '', 0 and NULL are all
    // the default group.
    if (empty($group)) {
      $group = 0;
    }

    // Check for a group.
    if (!isset($this->where[$group])) {
      $this->setWhereGroup('AND', $group);
    }

    $this->where[$group]['conditions'][] = [
      'field' => $field,
      'value' => $value,
      'operator' => $operator,
    ];
  }

  /**
   * Get Imago database id.
   *
   * @param string $database
   *   Database label.
   *
   * @return string
   *   Database id.
   */
  private function getDatabaseId(string $database) {
    $database_id = 'st';
    if ($database === 'sport') {
      $database_id = 'sp';
    }

    return $database_id;
  }

}
